//------------------------------------------------------------------------------------------------

//
// File: linesegment.cpp
// Author: Roman Lysecky
// Date Created: 28 November 2011
// 
// Description: Implementation of a LineSegment class.
// 
// Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2011.
// 
// 

//------------------------------------------------------------------------------------------------

#include <iostream>
#include <cmath>
#include <cstring>
#include "linesegment.h"

//------------------------------------------------------------------------------------------------

using namespace std;

//------------------------------------------------------------------------------------------------
//
// Simple constructor for LineSegment Class
//
LineSegment::LineSegment() {
    x1 = 0;
    y1 = 0;
    x2 = 0;
    y2 = 0;
    id = 0;
}

//------------------------------------------------------------------------------------------------
//
// Constructor with optional inputs for all LineSegment elements (x1, y1, x2, y2)
//
LineSegment::LineSegment(char *id_, float x1_, float y1_, float x2_, float y2_)
{    
    x1 = x1_;
    y1 = y1_;
    x2 = x2_;
    y2 = y2_;
     
    // Create a copy of the input string if it was provided.
    if( id_ == 0 ) id = 0;
    else {
        // USE THE FOLLOWING TO DEMONSTRATE A BAD_ALLOC EXCEPTION
        //id = new char[strlen(id_)+1-1000];
        
        id = new char[strlen(id_)+1];
        strcpy(id, id_);
    }
}

//------------------------------------------------------------------------------------------------

LineSegment::LineSegment(const LineSegment &fromLineSegment)
{
    x1 = fromLineSegment.x1;
    y1 = fromLineSegment.y1;
    x2 = fromLineSegment.x2;
    y2 = fromLineSegment.y2;  
    
    // Need to make a copy of any elements that are dynamically allocated. Otherwise,
    // mutliple instacnes of LineSegments will point to the same string for id, which would be 
    // dificult to track and lead to some problems with deleteing the id in the destructor.
    if( fromLineSegment.id == 0 ) id = 0;
    else {
        id = new char[strlen(fromLineSegment.id)+1];
        strcpy(id, fromLineSegment.id);
    }
}

//------------------------------------------------------------------------------------------------

LineSegment::~LineSegment() {
    if( id != 0 ) delete [] id;
}

//------------------------------------------------------------------------------------------------

LineSegment & 
LineSegment::operator=(const LineSegment &rhs)
{
    x1 = rhs.x1;
    y1 = rhs.y1;
    x2 = rhs.x2;
    y2 = rhs.y2;  
    
    // Need to make a copy of any elements that are dynamically allocated. Otherwise,
    // mutliple instacnes of LineSegments will point to the same string for id, which would be 
    // dificult to track and lead to some problems with deleteing the id in the destructor.
    if( rhs.id == 0 ) id = 0;
    else {
        id = new char[strlen(rhs.id)+1];
        strcpy(id, rhs.id);
    }

    return *this;
}

//------------------------------------------------------------------------------------------------

// Overload the < operator so we can sort a list of LineSegments.

bool 
operator<(const LineSegment& lhs, const LineSegment& rhs)
{
    int lhs_xmid, lhs_ymid;
    int rhs_xmid, rhs_ymid;
    int lhs_dist, rhs_dist;
    
    // calculate midpoint of lhs line segment
    lhs_xmid = (lhs.x1+lhs.x2)/2;
    lhs_ymid = (lhs.y1+lhs.y2)/2;
    
    // calculate distance to midpoint from origin (0,0) of lhs segment
    lhs_dist = sqrt((lhs_xmid*lhs_xmid)+(lhs_ymid*lhs_ymid));

    // calculate midpoint of rhs line segment
    rhs_xmid = (rhs.x1+rhs.x2)/2;
    rhs_ymid = (rhs.y1+rhs.y2)/2;

    // calculate distance to midpoint from origin (0,0) of rhs segment
    rhs_dist = sqrt((rhs_xmid*rhs_xmid)+(rhs_ymid*rhs_ymid));
    
    if( lhs_dist < rhs_dist ) return true;
    return false;
}

//------------------------------------------------------------------------------------------------

ostream & 
operator<< (ostream &os, LineSegment &lineSegment)
{
    os << "[" << lineSegment.id << "]" 
       << "(" << lineSegment.x1 << "," << lineSegment.y1 << ")->(" 
       << lineSegment.x2 << "," << lineSegment.y2 << ")";
    
    return os;
}

//------------------------------------------------------------------------------------------------

