//------------------------------------------------------------------------------------------------

//
// File: linesegment.h
// Author: Roman Lysecky
// Date Created: 28 November 2011
// 
// Description: Declaration of a LineSegment class with basis constructor and destructor.
// 
// Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2011.
// 
// 

//------------------------------------------------------------------------------------------------

#ifndef LINESEGMENT_H
#define LINESEGMENT_H

//------------------------------------------------------------------------------------------------

#include <iostream>

//------------------------------------------------------------------------------------------------

using namespace std;

//------------------------------------------------------------------------------------------------

class LineSegment {
    
public:
    // Declaration for constructor 
    LineSegment();
    LineSegment(char *id_, float x1_, float y1_, float x2_=0.0, float y2_=0.0);
    LineSegment(const LineSegment &fromLineSegment);

    // Declaration for destructor
    ~LineSegment();

    // Overload assignment operator 
    LineSegment & operator=(const LineSegment &rhs);

    // Overloading the < operator so we can sort a list of line segments
    friend bool operator<(const LineSegment& lhs, const LineSegment& rhs);

    // Overloading the output stream operator so we can display the line segment
    // using C++ stream such as cout
    friend ostream & operator<< (ostream &os, LineSegment &lineSegment);
    
public:
    float x1, y1;
    float x2, y2;
    char *id;
};

//------------------------------------------------------------------------------------------------

bool operator<(const LineSegment& lhs, const LineSegment& rhs);
ostream &operator<< (ostream &os, LineSegment &lineSegment);

//------------------------------------------------------------------------------------------------

#endif

//------------------------------------------------------------------------------------------------
